/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
// Modifications Complete 05/14/98 Dean Polla hummer@cs.uidaho.edu

// ***************************************************************************
// SclInterface Class
// 
//
// Class Description:  
// ------------------
// Purpose:  This class defines the layout and functionality of the Security Level
// Interface.
//
// This interface is a seperate window from the main interface.  This class 
// instantiates and lays out the GUI components for the Scl interface.  It 
// implements its own event handling and is capable of interacting with the whole 
// system.  If this window is destroyed, only it will be destroyed.  This interface 
// allows the user to add, remove, configure, assign, un-assign, and view security 
// classification levels.
//
// System Relationship:  Intercommunication with the whole system is minimal.  The
// majority of system relationships involves updating Lists or Choice components 
// located on other interfaces.  For example, updating the Security Levels List (SCL Types) 
// on the Main Interface or the Atom.class.  File input/output is performed.
//
// Requirements:  This class requires the Java Developemnt Kit (JDK) 1.1
// Aside from this there are no special algorithms, or constraints.  
//
// Pre-Conditions:  Atom.class.
// Post Conditions: True.
// Error Handling:  Addressed with the try and catch clause.
//
//
// Class Specification:
// --------------------
// This class must layout the interface components for the Scl Interface onto the 
// screen.  The Scl Interface appears on the console when the "SCL" button on the
// main interface is activated by the user.  This class provides these options:
// add, remove, configure, assignment, and view.  These options will be layed out
// in a tabular format for the user to choose from.  This class must respond to user
// generated events.  The following is a list of requirements used by this class:
//
//    *  Instantiate java.awt components used by the GUI
//    *  Layout the java.awt components
//    *  Provide event handling for the java.awt components
//
//
// Constructor
// -----------
//      public SclInterface()
//
// Methods
// -------
//      public void layoutAddPanel()
//      public void layoutRemovePanel()
//      public void layoutConfigurePanel()
//      public void layoutAssignmentPanel()
//      public void layoutViewPanel()
//
// Event Handling 
// --------------
//      public boolean actionPerformed(ActionEvent evt)
//      public boolean itemStateChanged(ItemEvent evt)
//
// External Classes Used
// ---------------------
//      FileIO        ToolExecution
//      Box           Constrain   
//      
// *****************************************************************************

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class SclInterface extends Constrain implements ActionListener, ItemListener {

  // *******************************************
  // Interface Component Declarations
  // *******************************************
  Panel   tabs;              // Sub-container for the buttons
  Panel   cards;             // Sub-container for the button features
  Panel   addPanel;          // Sub-container for the Add feature components
  Panel   removePanel;       // Sub-container for the Remove feature components
  Panel   configurePanel;    // Sub-container for the Configure feature components
  Panel   assignPanel;       // Sub-container for the Start feature components
  Panel   viewPanel;         // Sub-container for the Start feature components

  Button  addTab;            // Add tab
  Button  removeTab;         // Remove tab
  Button  configureTab;      // Configure tab
  Button  assignTab;         // Assign tab
  Button  viewTab;           // View tab

  Button addSubmitButton;          // Add Submit Button
  Button removeSubmitButton;       // Remove Submit Button
  Button configureSubmitButton;    // Configure Submit Button
  Button assignmentSubmitButton;   // Assign Submit Button

  TextField  addNameTextField;             // Add TextField for name
  TextField  assignExecTextField;          // Assignment TextField for Execution Command
  TextField  assignDirectoryTextField;     // Assignment TextField for Directory

  TextArea   addDescriptionTextArea;       // Add TextArea for Description
  TextArea   configureTextArea;            // Configure TextArea Current Configuration
  TextArea   configureTextAreaEdit;        // Configure TextArea for New Configuration
  TextArea   viewTextArea;                 // View TextArea

  List  removeSclList;                     // Remove Security Level List
  List  configureSclList;                  // Configure Security Level List
  public static List  assignToolList;      // Assignment Registered Tool List 
  List  assignSclList;                     // Assignment Security Level List
  List  viewSclList;                       // View Security Level List

  Choice    configureChoice;               // Configure Choice menu 

  Checkbox       assignCheckBox;           // Assignment Checkbox for assignment
  Checkbox       unassignCheckBox;         // Assignment Checkbox for un-assignment
  CheckboxGroup  assignmentGroup;          // Assignment Checkbox group

  CardLayout singleCard   = new CardLayout();     // Layout Manager for the feature / cards
  GridBagLayout  gridbag  = new GridBagLayout();  // Layout Manager for the components


  // *******************************************
  // Data Declarations
  // *******************************************
  private String[] alphaListSelection          = new String[0];  // List selection elements (alpha name)
  private String[] secondaryAlphaListSelection = new String[0];  // List selection elements (alpha name)
  private String   selectedItem;


  // *******************************************
  // Object instantiation
  // *******************************************
  FileIO sclIO      = new FileIO();                      

 
  // *****************************************************************************
  // public SclInterface()
  //
  // Constructor Description:  The constructor for the SclInterface  class.  Initializes
  // new variables and components, and lays out the components on the scl interface.
  //
  // Constructor Specification:  The constructor lays out the components for 
  // each area on the scl interface.  It uses the GridBagLayout Manager.  In addition
  // it defines the variables from the Data Declarations.  The following is a listing
  // of the interface layout and its corresponding components:
  // 
  //   Position  Description             Components
  //   --------  -----------             ----------
  //    * Frame  Scl Interface           GridBagLayout Manager
  //    * North  Tabs/Features Panel     GridLayout Manager, addTab, removeTab, 
  //                                     configureTab, assignTab, viewTab
  //    * Center Cards                   CardLayout Manager
  //             Add                     addPanel, addNameTextField, addDescriptionTextArea 
  //             Remove                  removePanel, removeSclList, panel1
  //             Configure               configurePanel, configureSclList, configureTextArea,
  //                                     configureTextAreaEdit, configureChoice, panel1, panel2
  //             Assignment              assignPanel, assignSclList, assignToolList, 
  //                                     assignExecTextField, assignDirectoryTextField,
  //                                     startCheckBox
  //             View                    viewSclList, TextArea
  //
  // The constructor will be accessed each time the SclInterface, is instantiated.
  // 
  // Pre-Conditions:  True.
  // Post-Conditions: True.
  //
  // *****************************************************************************

  public SclInterface () {
    setTitle("Security Level Options");

    addWindowListener(new WinAdapter());
    
    // *** Initialize Buttons ****
    addTab        = new Button("Add");
    removeTab     = new Button("Remove");
    configureTab  = new Button("Configure");
    assignTab     = new Button("Assignment");
    viewTab       = new Button("View");
    
    // *** BUTTON LISTENERS ***
    addTab.addActionListener(this);
    removeTab.addActionListener(this);
    configureTab.addActionListener(this);
    assignTab.addActionListener(this);
    viewTab.addActionListener(this);

    // *** Set Font and Color of Buttons ****
    addTab.setFont(new Font("Dialog", Font.BOLD, 14));
    removeTab.setFont(new Font("Dialog", Font.BOLD, 14));
    configureTab.setFont(new Font("Dialog", Font.BOLD, 14));
    assignTab.setFont(new Font("Dialog", Font.BOLD, 14));
    viewTab.setFont(new Font("Dialog", Font.BOLD, 14));

    // *** Tabs ****
    tabs = new Panel();
    tabs.setLayout(new GridLayout(0,6));
     tabs.add(addTab);
     tabs.add(removeTab);   
     tabs.add(configureTab);
     tabs.add(assignTab);
     tabs.add(viewTab);
    add("North", tabs);

    // *** Layout Card Method ****
    layoutAddPanel();

    // *** Cards ****
    cards = new Panel();
    cards.setLayout(singleCard);
    cards.add("Add",       addPanel);
    add("Center", cards);

  }  // Constructor


  // ***************************************************************************** 
  // public boolean actionPerformed(ActionEvent evt)
  //
  // Method Description:  This method handles all of the events of type ActionEvent
  // generated by the components in the scl interface.
  //
  // Method Specification:  If an ActionEvent occurs this method will handle it.
  // Events are generated by the user activating a component.  Such as, clicking a
  // button, or selecting a list item.  The following lists the events and reactions
  // for each ActionEvent:
  //
  //        Label     Component/Event        Reaction
  //        -----     ---------------        --------
  //      * Add        addSubmitButton       Add a security classification level to the
  //                                         scl_types.dat
  //      * Remove     removeSubmitButton    Remove a security classification level from 
  //                                         the scl_types.dat
  //      * Configure  configureSubmitButton Modify the user selected field of the security
  //                                         classification level
  //      * Assignment assignmentSubmitButton  Assign an audit tool to a security classification
  //                                           level
  //
  // This method will be accessed many times.
  //
  // Pre-Condition:  Component objects have been instantiated.
  // Post-Condition: True. Events handle properly.
  //
  // *****************************************************************************

  public void actionPerformed(ActionEvent evt) {
    Object source  = evt.getSource();


    // -----------------------------------------------------------------------
    // ADD
    //
    //   * Text Field: <name> 
    //   * Text Area:  <description>
    // -----------------------------------------------------------------------
    if (source == addSubmitButton) {
	String nameInfo        = addNameTextField.getText();
	String descriptionInfo = addDescriptionTextArea.getText();

	// VALIDITY check
	if (nameInfo.equals("")) {
	    Atom.systemStatusTextArea.append("The NAME field is required to Add a Security Classification Level.\n");
	}
	else {   
	    
	    // Append information to Atom.sclTypesFile 
	    sclIO.AtplAppendText(Atom.sclTypesFile, ToolInterface.NAME, nameInfo);
	    sclIO.AtplAppendText(Atom.sclTypesFile, ToolInterface.DESCRIPTION, descriptionInfo);
	    sclIO.AtplAppendText(Atom.sclTypesFile, ToolInterface.END, ToolInterface.ENDSYMBOL);

	    // UPDATE All SCL Lists
	    Atom.sclTypesList.add(nameInfo);
	
	    // The ToolInterface class may not have been instantiated yet, so
	    // check and handle these lists if possible.
	    try {
		ToolInterface.addChoice.add(nameInfo);
		ToolInterface.startChoice.add(nameInfo);
	    }
	    catch (NullPointerException e) {
		String err = e.toString();
	    }

	    // Clear the text fields
	    addNameTextField.setText("");
	    addDescriptionTextArea.setText("");
	}
    } // add



    // -----------------------------------------------------------------------
    // REMOVE
    //
    //   * Security Levels List 
    // -----------------------------------------------------------------------    
    if (source == removeSubmitButton) {

	// VALIDITY check
	if (selectedItem == null) {
	    Atom.systemStatusTextArea.append("Please select an item to Remove. \n");
	}
	else {

	    // UPDATE All SCL Lists
	    removeSclList.remove(selectedItem);
	    Atom.sclTypesList.remove(selectedItem);
	    
	    // The ToolInterface class may not have been instantiated yet, so
	    // check and handle these lists if possible.
	    try {
		ToolInterface.addChoice.remove(selectedItem);
		ToolInterface.startChoice.remove(selectedItem);
	    }
	    catch (NullPointerException e) {
		String err = e.toString();
	    }
	    
	    // UPDATE Atom.sclTypesFile
	    sclIO.AtplRemoveRecord(Atom.sclTypesFile, Atom.sclTempFile, selectedItem);	    

	    // FILE REMOVAL of Atom.sclTypesFile in ./levels/
	    selectedItem = selectedItem.replace(' ','_');
	    File removeFile = new File(Atom.sclLevelDir+selectedItem);	    
	    if (removeFile.exists()) {
		removeFile.delete();
	    }
	} // else
	selectedItem=null;
    } // remove
    

    // -----------------------------------------------------------------------
    // CONFIGURE
    //
    //   * Choice Menu: <name> <description>
    //   * Text Area: <new information>
    //   * Security Levels List
    // -----------------------------------------------------------------------
    if (source == configureSubmitButton) {      
	String choiceItem = configureChoice.getSelectedItem();
	String info       = configureTextAreaEdit.getText();           
	info = info.trim();

	// VALIDITY Check
	if (!(alphaListSelection.length > 0)) {
	    Atom.systemStatusTextArea.append("Please select an item from the list to configure.\n");
	    return;
	}

	// VALIDITY Check
	if (info.equals("")) {
	    Atom.systemStatusTextArea.append("Please enter New Configuration information \n");
	    return;
	}

	sclIO.AtplReplaceRecord(Atom.sclTypesFile, Atom.sclTempFile, alphaListSelection[0],
				choiceItem, info);
	
	// UPDATE all SCL lists
	if (choiceItem.equals(ToolInterface.NAME)) {
	    int index = configureSclList.getSelectedIndex();
	    Atom.sclTypesList.replaceItem(info,index);
	    configureSclList.replaceItem(info,index);
	    alphaListSelection[0] = info;
	}

	// DISPLAY
	configureTextArea.setText("");
	configureTextAreaEdit.setText("");
	sclIO.AtplGetRecord(Atom.sclTypesFile, alphaListSelection[0], configureTextArea);
    } // configure


    // -----------------------------------------------------------------------
    // ASSIGNMENT
    //
    //   * Registered Tools List
    //   * Security Level List
    //   * Text Field: <directory>
    //   * Text Field: <parameters>
    // -----------------------------------------------------------------------
    if (source  == assignmentSubmitButton) {             
	String execCommand  = assignExecTextField.getText();
	String dirInfo      = assignDirectoryTextField.getText();

      // VALIDITY check --> Text Fields
      if ((execCommand.equals("")) || (dirInfo.equals(""))) {
	  Atom.systemStatusTextArea.append("Please enter information in Directory and Execution Command Text Fields. \n");
	  return;
      }
	      
      // VALIDITY check --> Lists
      if (!((alphaListSelection.length > 0) && (secondaryAlphaListSelection.length > 0))) {
	  Atom.systemStatusTextArea.append("Please select items from BOTH lists.\n");
	  return;
      }
      
      if (assignmentGroup.getSelectedCheckbox().equals(assignCheckBox)) {
	  String processInfo = sclIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], 
						  ToolInterface.PROCESS);
	  sclIO.appendSclAssignment(secondaryAlphaListSelection[0], processInfo, execCommand, dirInfo);
      }
      else {
	  sclIO.removeSclAssignment(execCommand, secondaryAlphaListSelection[0]);	  
      }
    } // assignment


    // ------------------------------------------------------------
    // Cards
    // ------------------------------------------------------------

    if (source instanceof Component &&
	((Component)evt.getSource()).getParent().equals(tabs)) {

      String tabType = evt.getActionCommand();
       
      //
      // If the user select a tab/feature display the apporpriate card, moreover
      // display the card/features interface
      //
      if (tabType.equals("Remove")) {
	layoutRemovePanel();
	selectedItem=null;
	cards.add("Remove",    removePanel);
      }
    
      if (tabType.equals("Configure")) {
	layoutConfigurePanel();
	cards.add("Configure",    configurePanel);
	alphaListSelection = new String[0];
	secondaryAlphaListSelection = new String[0];
      }       

      if (tabType.equals("Assignment")) {
	layoutAssignmentPanel();
	cards.add("Assignment",    assignPanel);
	alphaListSelection = new String[0];
	secondaryAlphaListSelection = new String[0];
      }       

      if (tabType.equals("View")) {
	layoutViewPanel();
	cards.add("View", viewPanel);
	alphaListSelection = new String[0];
      }       
       
      singleCard.show(cards, tabType);
    }
  } // action



class WinAdapter extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
    dispose();
  }
}


  // *********************************************************************************
  // public boolean ItemStateChanged(ItemEvent evt)
  //
  // Method Description:  This method handles all of the events that are of type
  // ItemEvent.  These include list selection, and list deselection
  //
  // Method Specification:  If one of the preceeding events occurs this method will 
  // handle it.  Events are generated by the user activating a component.  Such as, 
  // selecting or deselecting list elements.
  //
  //        Action Type     Component/Event        Reaction
  //        -----------     ---------------        --------
  //      * LIST_SELECT     removeSclList          alphalistSelection = selected item
  //                        configureSclList       alphaListSelection = selected item
  //                        assignToolList         alphaListSelection = selected item
  //                        assignSclList          secondaryAlphaListSelection = selected item
  //      * LIST_DESELECT   configureSclList       alphaListSelection = selected item
  //                        assignToolList         alphaListSelection = selected item
  //                        assignSclList          secondaryAlphaListSelection = selected item
  //
  // This method will be accessed many times. 
  //
  // Pre-Condition:  Component objects have been instantiated.
  // Post-Condition: True.  Lists have proper component names or indexes.
  // 
  // *********************************************************************************

  public void itemStateChanged(ItemEvent evt) {
    String fieldData = "";
    String dirData = "";

    switch(evt.getStateChange()) {
    case ItemEvent.SELECTED:

      // Remove
      if (evt.getSource().equals(removeSclList)) {
	  selectedItem = removeSclList.getSelectedItem();
      }
      
      // Configure
      //     * Get List selection
      //     * Clear TextField
      //     * Place record information in TextField based on List selection
      //
      if (evt.getSource().equals(configureSclList)) {
	  alphaListSelection = configureSclList.getSelectedItems();
	  alphaListSelection[0] = alphaListSelection[0].replace('\n', ' ');
	  configureTextArea.setText("");
	  sclIO.AtplGetRecord(Atom.sclTypesFile, alphaListSelection[0], configureTextArea);
      }

      // Assignment
      //     * Get List selection
      //     * Clear TextFields
      //     * Set Textfield information
      //
      if (evt.getSource().equals(assignToolList)) {
	  alphaListSelection = assignToolList.getSelectedItems();
	  assignExecTextField.setText("");
	  assignDirectoryTextField.setText("");
	  fieldData = sclIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], 
					 ToolInterface.DEFAULT);
	  dirData   = sclIO.AtplGetField(Atom.auditToolsFile, alphaListSelection[0], 
					 ToolInterface.DIRECTORY);	
	  assignExecTextField.setText(fieldData);
	  assignDirectoryTextField.setText(dirData);
      }

      if (evt.getSource().equals(assignSclList)) {
	  secondaryAlphaListSelection = assignSclList.getSelectedItems();
      }

      // View
      //    * Get List selection
      //    * Clear TextField
      //    * Place information in TextField
      //
      if (evt.getSource().equals(viewSclList)) {
	  alphaListSelection = viewSclList.getSelectedItems();
	  viewTextArea.setText("");
	  sclIO.sclGetText(alphaListSelection[0], viewTextArea);
      }
      break;
      
    case ItemEvent.DESELECTED:
      
      // Configure
      if (evt.getSource().equals(configureSclList)) {
	  alphaListSelection = configureSclList.getSelectedItems();
      }
      
      // Assign
      if (evt.getSource().equals(assignToolList)) {
	  alphaListSelection = assignToolList.getSelectedItems();
      }

      if (evt.getSource().equals(assignSclList)) {
	  secondaryAlphaListSelection = assignSclList.getSelectedItems();
      }
      
      break;
    }
    
  } // handleEvent
    

  // *********************************************************************************
  // public void layoutAddPanel ()
  //
  // Method Description:  This method lays out all of the components for the Add tab,
  // located on the scl interface.
  //
  // Method Specification:  When the Add tab is activated by the user, the corresponding
  // Add feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Add tab is activated this
  // method will be called.
  //
  // Pre-Condition:  Add components have been declared.
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutAddPanel () {
    addPanel                 = new Panel();
    addNameTextField         = new TextField(40);
    addDescriptionTextArea   = new TextArea(16,60);

    addPanel = new Panel();    
    addPanel.setLayout(gridbag);
    addSubmitButton = new Button("Submit");
    addSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    addSubmitButton.setForeground(Color.blue);

    // ADD LISTENER
    addSubmitButton.addActionListener(this);

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager. 
    constrain(addPanel, new Label("Name:"), 0,0,1,1, 20,20,0,0);
    constrain(addPanel, addNameTextField, 0,1,1,1, 0,20,0,0);     
    constrain(addPanel, new Label("Description:"), 0,2,1,1, 10,20,0,0);
    constrain(addPanel, addDescriptionTextArea, 0,3,1,1, 0,20,0,0);
    constrain(addPanel, addSubmitButton, 0,4,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 20, 0, 5, 5);   

  } // layoutAddPanel


  // *********************************************************************************
  // public void layoutRemovePanel ()
  //
  // Method Description:  This method lays out all of the components for the Remove tab,
  // located on the scl interface.
  //
  // Method Specification:  When the Remove tab is activated by the user, the corresponding
  // Remove feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Remove tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Remove components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutRemovePanel() {
    removePanel         = new Panel();            
    removeSclList       = new List(20, false);       
    Panel panel1        = new Panel();

    panel1.setLayout(gridbag);
    removePanel.setLayout(gridbag);
    removeSubmitButton = new Button("Submit");
    removeSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    removeSubmitButton.setForeground(Color.blue);   
    
    // ADD LISTENERS
    removeSubmitButton.addActionListener(this);
    removeSclList.addItemListener(this);

    // Fill the removeSclList with the names of the Scl types 
    removeSclList = sclIO.fillList(removeSclList, Atom.sclTypesFile, ToolInterface.NAME);
    
    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.      
    constrain(panel1, removeSclList, 0,0,1,1);
    Box panelBox = new Box(panel1, "Security Levels");
    panelBox.etchedOut();
    
    constrain(removePanel, panelBox, 0,0,1,1);
    constrain(removePanel, removeSubmitButton, 0,1,1,1, 20,40,0,0);
     
  } // layoutRemovePanel


  // *********************************************************************************
  // public void layoutConfigurePanel ()
  //
  // Method Description:  This method lays out all of the components for the Configure tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Configure tab is activated by the user, the corresponding
  // Configure feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Configure tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Configure components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutConfigurePanel() {
    configurePanel              = new Panel();
    configureSclList            = new List(22, false);
    configureTextArea           = new TextArea(8,50);
    configureTextArea.setEditable(false);
    configureTextAreaEdit       = new TextArea(8,50);
    configureChoice             = new Choice();

    Panel     panel1            = new Panel();
    Panel     panel2            = new Panel(); 
       
    configurePanel.setLayout(gridbag);
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);
    configureSubmitButton = new Button("Submit");
    configureSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
    configureSubmitButton.setForeground(Color.blue);

    // ADD LISTENERS
    configureSubmitButton.addActionListener(this);
    configureSclList.addItemListener(this);

    // Fill the configureSclList with the names of the Scl types 
    configureSclList = sclIO.fillList(configureSclList, Atom.sclTypesFile, ToolInterface.NAME);

    // This is where the Choice menu is filled with choices
    configureChoice.add("NAME");
    configureChoice.add("DESCRIPTION");
    configureChoice.setFont(new Font("Courier", Font.PLAIN, 12));

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.      
    constrain(panel1, configureSclList, 0,0,1,1);
    Box panelBox = new Box(panel1, "Security Levels");
    panelBox.etchedOut();

    constrain(panel2, configureChoice, 0,0,1,1, 10,0,0,0);
    constrain(panel2, new Label("Current Configuration"), 0,1,1,1, 20,0,0,0);
    constrain(panel2, configureTextArea, 0,2,1,1);
    constrain(panel2, new Label("New Configuration"), 0,3,1,1, 10,0,0,0);
    constrain(panel2, configureTextAreaEdit, 0,4,1,1); 
    constrain(panel2, configureSubmitButton, 0,5,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

    constrain(configurePanel, panelBox, 0,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 20, 10, 5, 10);
    constrain(configurePanel, panel2, 1,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 20, 0, 5, 10);
    
  } // layoutConfigurePanel


  // *********************************************************************************
  // public void layoutAssignmentPanel ()
  //
  // Method Description:  This method lays out all of the components for the Assign tab,
  // located on the tool interface.
  //
  // Method Specification:  When the Assign tab is activated by the user, the corresponding
  // Assign feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the Assign tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  Assign components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutAssignmentPanel() {
    assignPanel                     = new Panel();
    assignSclList                   = new List(15, false);
    assignToolList                  = new List(15, false);
    assignExecTextField       = new TextField(60);
    assignDirectoryTextField        = new TextField(60);
    assignmentGroup                 = new CheckboxGroup();
    assignCheckBox                  = new Checkbox("Assign Registered Tool", assignmentGroup, true);
    unassignCheckBox                = new Checkbox("Un-assign Registered Tool", assignmentGroup, false);

    Panel   insetPanelA             = new Panel();
    Panel   insetPanelB             = new Panel();
    Panel   insetPanelC             = new Panel();
    Panel   panel1                  = new Panel();
    Panel   panel2                  = new Panel();

    Label   descr1                  = new Label("Un-assignment will remove all");
    Label   descr2                  = new Label("instances of Registered Tool");
    
    insetPanelA.setLayout(new FlowLayout());
    insetPanelB.setLayout(new FlowLayout());
    insetPanelC.setLayout(gridbag);

    assignPanel.setLayout(gridbag);
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);
    
     assignmentSubmitButton = new Button("Submit");
     assignmentSubmitButton.setFont(new Font("Dialog", Font.BOLD, 12));
     assignmentSubmitButton.setForeground(Color.blue);

     assignCheckBox.setFont(new Font("Courier", Font.PLAIN, 12));
     unassignCheckBox.setFont(new Font("Courier", Font.PLAIN, 12));

     descr1.setForeground(Color.red);
     descr2.setForeground(Color.red);

     // ADD LISTENERS
     assignmentSubmitButton.addActionListener(this);
     assignToolList.addItemListener(this);
     assignSclList.addItemListener(this);

     // Fill the assignToolList with the names of the registered audit tools 
     // Fill the assignSclList with the names of the Scl types 
     assignToolList = sclIO.fillList(assignToolList, Atom.auditToolsFile, ToolInterface.NAME);
     assignSclList  = sclIO.fillList(assignSclList,  Atom.sclTypesFile, ToolInterface.NAME);
     
     // Layout Registered Audit Tools and Security Levels Lists
     insetPanelA.add(assignToolList);
     Box panelABox = new Box(insetPanelA, "Registered Tools");
     panelABox.etchedOut();

     insetPanelB.add(assignSclList);
     Box panelBBox = new Box(insetPanelB, "Security Levels");
     panelBBox.etchedOut();
     
     constrain(insetPanelC, assignCheckBox, 0,0,1,1, 10,0,0,0);
     constrain(insetPanelC, unassignCheckBox, 0,1,1,1, 10,0,0,0);
     constrain(insetPanelC, descr1, 0,2,1,1, 15,0,0,0);
     constrain(insetPanelC, descr2, 0,3,1,1);

     constrain(panel1, panelABox, 0,0,1,1, 0,0,0,0);
     constrain(panel1, panelBBox, 1,0,1,1, 0,20,0,0);
     constrain(panel1, insetPanelC, 2,0,1,1, 0,20,0,0);

     constrain(panel2, new Label("Directory:"), 0,0,1,1);
     constrain(panel2, assignDirectoryTextField, 0,1,1,1);
     constrain(panel2, new Label("Execution Command:"), 0,2,1,1);
     constrain(panel2, assignExecTextField, 0,3,1,1);
     constrain(panel2, assignmentSubmitButton, 0,4,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.CENTER, 1.0, 1.0, 10, 0, 5, 5);

     constrain(assignPanel, panel1, 0,0,1,1, GridBagConstraints.NONE,
	       GridBagConstraints.NORTH, 1.0, 1.0, 10, 0, 0, 0);
     constrain(assignPanel, panel2, 0,1,1,1, GridBagConstraints.HORIZONTAL,
	       GridBagConstraints.SOUTH, 1.0, 1.0, 10, 20, 0, 0);
         
  } // layoutAssignmentPanel



  // *********************************************************************************
  // public void layoutViewPanel ()
  //
  // Method Description:  This method lays out all of the components for the View tab,
  // located on the tool interface.
  //
  // Method Specification:  When the View tab is activated by the user, the corresponding
  // View feature interface needs to be displayed.  This method instantiates all of the 
  // components and positions them using the GridBagLayout manager.
  //   
  // This method will be accessed several times.  Every time the View tab is activated 
  // this method will be called.
  //
  // Pre-Condition:  View components have been declared in the Interface Components 
  // Declarations.
  //
  // Post-Condition: True.  The components have been instantiated and layed out.
  // 
  // *********************************************************************************

  public void layoutViewPanel() {
    viewPanel                    = new Panel();
    viewSclList                  = new List(20, false);
    viewTextArea                 = new TextArea(23,50);
    viewTextArea.setEditable(false);
    Panel     panel1             = new Panel();
    Panel     panel2             = new Panel();

    viewPanel.setLayout(gridbag);
    panel1.setLayout(gridbag);
    panel2.setLayout(gridbag);

    // ADD LISTENERS
    viewSclList.addItemListener(this);
    
    // Fill the viewToolList with the names of the registered audit tools   
    viewSclList = sclIO.fillList(viewSclList, Atom.sclTypesFile, ToolInterface.NAME);     

    // The constrain method places the components onto the interface using the
    // GridBagLayout manager.                        
    constrain(panel1, viewSclList, 0,0,1,1);
    Box panelBox = new Box(panel1, "Security Levels");
    panelBox.etchedOut();
    
    constrain(panel2, new Label("Security Level Assignment"), 0,0,1,1);
    constrain(panel2, viewTextArea, 0,1,1,1);    
    
    constrain(viewPanel, panelBox, 0,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 20, 0, 0, 0);
    constrain(viewPanel, panel2, 1,0,1,1, GridBagConstraints.NONE,
	      GridBagConstraints.NORTH, 1.0, 1.0, 20, 10, 0, 10);

  }  // layoutViewPanel()

}












 



